package com.boman.file.controller;

import com.alibaba.fastjson.JSONObject;
import com.boman.domain.dto.AjaxResult;
import com.boman.domain.dto.Base64Dto;
import com.boman.file.utils.HttpClientUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import com.boman.domain.dto.R;
import com.boman.common.core.utils.file.FileUtils;
import com.boman.file.service.ISysFileService;
import com.boman.domain.SysFile;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * 文件请求处理
 *
 * @author ruoyi
 */
@RestController
public class SysFileController {
    private static final Logger log = LoggerFactory.getLogger(SysFileController.class);

    @Autowired
    private ISysFileService sysFileService;

    /**
     * 文件上传请求
     */
    @PostMapping("upload")
    public R<SysFile> upload(MultipartFile file) {
        try {
            // 上传并返回访问地址
            String url = sysFileService.uploadFile(file);
            SysFile sysFile = new SysFile();
            sysFile.setName(FileUtils.getName(url));
            sysFile.setUrl(url);
            return R.ok(sysFile);
        } catch (Exception e) {
            log.error("上传文件失败", e);
            return R.fail(e.getMessage());
        }
    }

    /**
     * 通用上传请求 暂且如此处理，后来再优化
     */
    @PostMapping("/upload/base64")
    public AjaxResult uploadFileBase64(@RequestBody Base64Dto dto) {
        try {
            String base64 = dto.getFile();
            String url = sysFileService.uploadFileBase64(base64);
            AjaxResult ajax = AjaxResult.success();
            ajax.put("name", FileUtils.getName(url));
            ajax.put("url", url);
            return ajax;
        } catch (Exception e) {
            return AjaxResult.error(e.getMessage());
        }
    }

    @GetMapping("/getConfigPath")
    public String getConfigPath() {
        return sysFileService.getConfigPath();
    }


    /**
     * ocr身份证识别,微信接口
     *
     * @return
     */
    @PostMapping("/ocrIdCard")
    public AjaxResult ocrIdCard(MultipartFile file) {

        String accessTokenResult = getAccessToken();
        JSONObject jsonObject = JSONObject.parseObject(accessTokenResult);
        String accessToken = jsonObject.getString("access_token");
        //https://api.weixin.qq.com/cv/ocr/idcard?type=MODE&img_url=ENCODE_URL&access_token=ACCESS_TOCKEN
        String result = "";
        try {
            String url = sysFileService.uploadFile(file);
            result = HttpClientUtils.doPost("https://api.weixin.qq.com/cv/ocr/idcard?type=photo&img_url=" + url + "&access_token=" + accessToken, null);
            JSONObject parse = JSONObject.parseObject(result);
            String errcode = parse.getString("errcode");
            System.out.println("解析身份证返回值:"+result);
            if ("0".equals(errcode)){
                return AjaxResult.success(parse);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.error("识别失败");
    }

    private static String getAccessToken() {
        String accessToken = "";
        try {
            accessToken = HttpClientUtils.doGet1("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=wxb9b83f3c86545690&secret=95adc6921a24a3c6cff55f2a1290f6f6");
        } catch (IOException e) {
            e.printStackTrace();
        }
        return accessToken;
    }
}